@@ -24,12 +24,17 @@ function clearStack()
 }
 
 function openApplicationListSource(origin) {
-    openApplicationList({ originFilter: origin, title: origin })
+    openApplicationList({ originFilter: origin, title: origin, allBackends: true })
+}
+
+function openUrlResources(uri) {
+    clearStack()
+    openApplicationList({ resourcesUrl: uri, title: uri, allBackends: true })
 }
 
 function openApplicationMime(mime) {
     clearStack()
-    openApplicationList({ mimeTypeFilter: mime , title: i18n("Resources for '%1'", mime) })
+    openApplicationList({ mimeTypeFilter: mime, title: i18n("Resources for '%1'", mime) })
 }
 
 function openApplicationList(props) {
@@ -56,7 +61,8 @@ function openExtends(ext) {
 }
 
 function openHome() {
-    window.globalDrawer.resetMenu();
+    if (window.globalDrawer.currentSubMenu)
+        window.globalDrawer.resetMenu();
     clearStack()
     window.globalDrawer.clearSearch();
     window.stack.push(topBrowsingComp)
