@@ -20,6 +20,7 @@
 
 #include "Transaction.h"
 
+#include <resources/AbstractResource.h>
 #include "TransactionModel.h"
 #include <QDebug>
 
@@ -35,6 +36,13 @@ Transaction::Transaction(QObject *parent, AbstractResource *resource,
 {
 }
 
+Transaction::~Transaction()
+{
+    if(status()<DoneStatus || TransactionModel::global()->contains(this)) {
+        qWarning() << "destroying Transaction before it's over" << this;
+    }
+}
+
 AbstractResource *Transaction::resource() const
 {
     return m_resource;
@@ -71,8 +79,10 @@ void Transaction::setStatus(Status status)
         m_status = status;
         emit statusChanged(m_status);
 
-        if (m_status == DoneStatus) {
+        if (m_status == DoneStatus || m_status == CancelledStatus || m_status == DoneWithErrorStatus) {
             setCancellable(false);
+
+            TransactionModel::global()->removeTransaction(this);
         }
     }
 }
@@ -98,3 +108,26 @@ bool Transaction::isActive() const
 {
     return m_status == DownloadingStatus || m_status == CommittingStatus;
 }
+
+QString Transaction::name() const
+{
+    return m_resource->name();
+}
+
+QVariant Transaction::icon() const
+{
+    return m_resource->icon();
+}
+
+bool Transaction::isVisible() const
+{
+    return m_visible;
+}
+
+void Transaction::setVisible(bool visible)
+{
+    if (m_visible != visible) {
+        m_visible = visible;
+        Q_EMIT visibleChanged(visible);
+    }
+}
