@@ -71,7 +71,7 @@ QVariant TransactionModel::data(const QModelIndex &index, int role) const
     if (!index.isValid())
         return QVariant();
 
-    Transaction *trans = transactionFromIndex(index);
+    Transaction *trans = m_transactions[index.row()];
     switch (role) {
     case TransactionRoleRole:
         return trans->role();
@@ -101,6 +101,10 @@ QVariant TransactionModel::data(const QModelIndex &index, int role) const
             break;
         case Transaction::DoneStatus:
             return i18nc("@info:status", "Done");
+        case Transaction::DoneWithErrorStatus:
+            return i18nc("@info:status", "Failed");
+        case Transaction::CancelledStatus:
+            return i18nc("@info:status", "Cancelled");
         }
         break;
     case TransactionRole:
@@ -112,16 +116,6 @@ QVariant TransactionModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
-Transaction *TransactionModel::transactionFromIndex(const QModelIndex &index) const
-{
-    Transaction *trans = nullptr;
-
-    if (index.isValid() && index.row() < m_transactions.size())
-        trans = m_transactions.at(index.row());
-
-    return trans;
-}
-
 Transaction *TransactionModel::transactionFromResource(AbstractResource *resource) const
 {
     Transaction *ret = nullptr;
@@ -175,13 +169,6 @@ void TransactionModel::addTransaction(Transaction *trans)
     emit transactionAdded(trans);
 }
 
-void TransactionModel::cancelTransaction(Transaction *trans)
-{
-    removeTransaction(trans);
-
-    emit transactionCancelled(trans);
-}
-
 void TransactionModel::removeTransaction(Transaction *trans)
 {
     Q_ASSERT(trans);
@@ -215,7 +202,7 @@ int TransactionModel::progress() const
     int sum = 0;
     int count = 0;
     foreach(Transaction* t, m_transactions) {
-        if (t->isActive()) {
+        if (t->isActive() && t->isVisible()) {
             sum += t->progress();
             ++count;
         }
