@@ -21,7 +21,6 @@
 
 #include "FlatpakBackend.h"
 #include "FlatpakFetchDataJob.h"
-#include "FlatpakFetchUpdatesJob.h"
 #include "FlatpakResource.h"
 #include "FlatpakSourcesBackend.h"
 #include "FlatpakTransaction.h"
@@ -29,14 +28,13 @@
 #include <resources/StandardBackendUpdater.h>
 #include <resources/SourcesModel.h>
 #include <Transaction/Transaction.h>
-#include <Transaction/TransactionModel.h>
 #include <appstream/OdrsReviewsBackend.h>
 #include <appstream/AppStreamIntegration.h>
 
 #include <AppStreamQt/bundle.h>
 #include <AppStreamQt/component.h>
 #include <AppStreamQt/icon.h>
-#include <AppStream/appstream.h>
+#include <AppStreamQt/metadata.h>
 
 #include <KAboutData>
 #include <KLocalizedString>
@@ -45,15 +43,18 @@
 #include <KSharedConfig>
 
 #include <QAction>
+#include <QtConcurrentRun>
 #include <QDebug>
 #include <QDir>
 #include <QFile>
+#include <QFutureWatcher>
 #include <QMessageBox>
 #include <QSettings>
 #include <QThread>
 #include <QTimer>
 #include <QTextStream>
 #include <QTemporaryFile>
+#include <QNetworkAccessManager>
 
 #include <glib.h>
 
@@ -64,6 +65,7 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_reviews(AppStreamIntegration::global()->reviews())
     , m_fetching(false)
+    , m_threadPool(new QThreadPool(this))
 {
     g_autoptr(GError) error = nullptr;
     m_cancellable = g_cancellable_new();
@@ -87,6 +89,7 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
 
 FlatpakBackend::~FlatpakBackend()
 {
+    m_threadPool.clear();
     for(auto inst : m_installations)
         g_object_unref(inst);
 
@@ -110,6 +113,55 @@ void FlatpakBackend::announceRatingsReady()
     }
 }
 
+class FlatpakFetchRemoteResourceJob : public QNetworkAccessManager
+{
+Q_OBJECT
+public:
+    FlatpakFetchRemoteResourceJob(const QUrl &url, FlatpakBackend *backend)
+        : QNetworkAccessManager(backend)
+        , m_backend(backend)
+        , m_url(url)
+    {
+    }
+
+    void start()
+    {
+        auto replyGet = get(QNetworkRequest(m_url));
+
+        connect(replyGet, &QNetworkReply::finished, this, [this, replyGet] {
+            const QUrl originalUrl = replyGet->request().url();
+            if (replyGet->error() != QNetworkReply::NoError) {
+                qWarning() << "couldn't download" << originalUrl << replyGet->errorString();
+                Q_EMIT jobFinished(false, nullptr);
+                return;
+            }
+
+            const QUrl fileUrl = QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1Char('/') + originalUrl.fileName());
+            auto replyPut = put(QNetworkRequest(fileUrl), replyGet->readAll());
+            connect(replyPut, &QNetworkReply::finished, this, [this, originalUrl, fileUrl, replyPut]() {
+                if (replyPut->error() == QNetworkReply::NoError) {
+                    auto res = m_backend->resourceForFile(fileUrl);
+                    if (res) {
+                        FlatpakResource *resource = qobject_cast<FlatpakResource*>(res);
+                        resource->setResourceFile(originalUrl);
+                        Q_EMIT jobFinished(true, resource);
+                    } else {
+                        qWarning() << "couldn't download" << originalUrl << "into" << fileUrl << replyPut->errorString();
+                        Q_EMIT jobFinished(false, nullptr);
+                    }
+                }
+            });
+        });
+    }
+
+Q_SIGNALS:
+    void jobFinished(bool success, FlatpakResource *resource);
+
+private:
+    FlatpakBackend *m_backend;
+    QUrl m_url;
+};
+
 FlatpakRemote * FlatpakBackend::getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const
 {
     auto remotes = flatpak_installation_list_remotes(installation, m_cancellable, nullptr);
@@ -128,10 +180,10 @@ FlatpakRemote * FlatpakBackend::getFlatpakRemoteByUrl(const QString &url, Flatpa
     return nullptr;
 }
 
-FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
+FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource) const
 {
-    AppStream::Component *component = resource->appstreamComponent();
-    AppStream::Component::Kind appKind = component->kind();
+    AppStream::Component component = resource->appstreamComponent();
+    AppStream::Component::Kind appKind = component.kind();
     FlatpakInstalledRef *ref = nullptr;
     GPtrArray *installedApps = nullptr;
     g_autoptr(GError) localError = nullptr;
@@ -142,9 +194,9 @@ FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation
 
     ref = flatpak_installation_get_installed_ref(flatpakInstallation,
                                                  resource->type() == FlatpakResource::DesktopApp ? FLATPAK_REF_KIND_APP : FLATPAK_REF_KIND_RUNTIME,
-                                                 resource->flatpakName().toStdString().c_str(),
-                                                 resource->arch().toStdString().c_str(),
-                                                 resource->branch().toStdString().c_str(),
+                                                 resource->flatpakName().toUtf8().constData(),
+                                                 resource->arch().toUtf8().constData(),
+                                                 resource->branch().toUtf8().constData(),
                                                  m_cancellable, &localError);
 
     // If we found installed ref this way, we can return it
@@ -173,7 +225,7 @@ FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation
     return ref;
 }
 
-FlatpakResource * FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref)
+FlatpakResource * FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const
 {
     foreach (FlatpakResource *resource, m_resources) {
         if (compareAppFlatpakRef(flatpakInstallation, resource, ref)) {
@@ -184,7 +236,7 @@ FlatpakResource * FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *fla
     return nullptr;
 }
 
-FlatpakResource * FlatpakBackend::getRuntimeForApp(FlatpakResource *resource)
+FlatpakResource * FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) const
 {
     FlatpakResource *runtime = nullptr;
     const auto runtimeInfo = resource->runtime().split(QLatin1Char('/'));
@@ -213,9 +265,9 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
     g_autoptr(GError) localError = nullptr;
     g_autoptr(GFile) file = nullptr;
     g_autoptr(FlatpakBundleRef) bundleRef = nullptr;
-    AppStream::Component *asComponent = nullptr;
+    AppStream::Component asComponent;
 
-    file = g_file_new_for_path(url.toLocalFile().toStdString().c_str());
+    file = g_file_new_for_path(url.toLocalFile().toUtf8().constData());
     bundleRef = flatpak_bundle_ref_new(file, &localError);
 
     if (!bundleRef) {
@@ -249,24 +301,23 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
         gconstpointer data = g_bytes_get_data(appstream, &len);
         g_autofree gchar *appstreamContent = g_strndup((char*)data, len);
 
-        g_autoptr(AsMetadata) metadata = as_metadata_new();
-        as_metadata_set_format_style(metadata, AS_FORMAT_STYLE_COLLECTION);
-        as_metadata_parse(metadata, appstreamContent, AS_FORMAT_KIND_XML, &localError);
-        if (localError) {
-            qWarning() << "Failed to parse appstream metadata:" << localError->message;
+        AppStream::Metadata metadata;
+        metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
+        AppStream::Metadata::MetadataError error = metadata.parse(QString::fromUtf8(appstreamContent), AppStream::Metadata::FormatKindXml);
+        if (error != AppStream::Metadata::MetadataErrorNoError) {
+            qWarning() << "Failed to parse appstream metadata: " << error;
             return nullptr;
         }
 
-        g_autoptr(GPtrArray) components = as_metadata_get_components(metadata);
-        if (g_ptr_array_index(components, 0)) {
-            asComponent = new AppStream::Component(AS_COMPONENT(g_ptr_array_index(components, 0)));
+        QList<AppStream::Component> components = metadata.components();
+        if (components.size()) {
+            asComponent = AppStream::Component(components.first());
         } else {
             qWarning() << "Failed to parse appstream metadata";
             return nullptr;
         }
     } else {
-        AsComponent *component = as_component_new();
-        asComponent = new AppStream::Component(component);
+        asComponent = AppStream::Component();
         qWarning() << "No appstream metadata in bundle";
     }
 
@@ -278,6 +329,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
     metadata = flatpak_bundle_ref_get_metadata(bundleRef);
     QByteArray metadataContent = QByteArray((char *)g_bytes_get_data(metadata, &len));
     if (!updateAppMetadata(resource, metadataContent)) {
+        delete resource;
         qWarning() << "Failed to update metadata from app bundle";
         return nullptr;
     }
@@ -296,12 +348,14 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
         resource->setBundledIcon(pixmap);
     }
 
+    const QString origin = QString::fromUtf8(flatpak_bundle_ref_get_origin(bundleRef));
+
     resource->setDownloadSize(0);
     resource->setInstalledSize(flatpak_bundle_ref_get_installed_size(bundleRef));
     resource->setPropertyState(FlatpakResource::DownloadSize, FlatpakResource::AlreadyKnown);
     resource->setPropertyState(FlatpakResource::InstalledSize, FlatpakResource::AlreadyKnown);
     resource->setFlatpakFileType(QStringLiteral("flatpak"));
-    resource->setOrigin(QString::fromUtf8(flatpak_bundle_ref_get_origin(bundleRef)));
+    resource->setOrigin(origin.isEmpty() ? i18n("Local bundle") : origin);
     resource->setResourceFile(url);
     resource->setState(FlatpakResource::None);
     resource->setType(FlatpakResource::DesktopApp);
@@ -338,30 +392,67 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
 
     auto ref = FLATPAK_REF(remoteRef);
 
-    AsComponent *component = as_component_new();
-    as_component_add_url(component, AS_URL_KIND_HOMEPAGE, settings.value(QStringLiteral("Flatpak Ref/Homepage")).toString().toStdString().c_str());
-    as_component_set_description(component, settings.value(QStringLiteral("Flatpak Ref/Description")).toString().toStdString().c_str(), nullptr);
-    as_component_set_name(component, settings.value(QStringLiteral("Flatpak Ref/Title")).toString().toStdString().c_str(), nullptr);
-    as_component_set_summary(component, settings.value(QStringLiteral("Flatpak Ref/Comment")).toString().toStdString().c_str(), nullptr);
+    AppStream::Component asComponent;
+    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Ref/Homepage")).toString());
+    asComponent.setDescription(settings.value(QStringLiteral("Flatpak Ref/Description")).toString());
+    asComponent.setName(settings.value(QStringLiteral("Flatpak Ref/Title")).toString());
+    asComponent.setSummary(settings.value(QStringLiteral("Flatpak Ref/Comment")).toString());
+    asComponent.setId(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+
     const QString iconUrl = settings.value(QStringLiteral("Flatpak Ref/Icon")).toString();
     if (!iconUrl.isEmpty()) {
-        AsIcon *icon = as_icon_new();
-        as_icon_set_kind(icon, AS_ICON_KIND_REMOTE);
-        as_icon_set_url(icon, iconUrl.toStdString().c_str());
-        as_component_add_icon(component, icon);
+        AppStream::Icon icon;
+        icon.setKind(AppStream::Icon::KindRemote);
+        icon.setUrl(QUrl(iconUrl));
+        asComponent.addIcon(icon);
     }
 
-    AppStream::Component *asComponent = new AppStream::Component(component);
     auto resource = new FlatpakResource(asComponent, preferredInstallation(), this);
     resource->setFlatpakFileType(QStringLiteral("flatpakref"));
     resource->setOrigin(QString::fromUtf8(remoteName));
     resource->updateFromRef(ref);
-    addResource(resource);
+
+    QUrl runtimeUrl = QUrl(settings.value(QStringLiteral("Flatpak Ref/RuntimeRepo")).toString());
+    if (!runtimeUrl.isEmpty()) {
+        auto installation = preferredInstallation();
+        // We need to fetch metadata to find information about required runtime
+        auto fw = new QFutureWatcher<QByteArray>(this);
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, installation, resource));
+        connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, installation, resource, fw, runtimeUrl]() {
+            const auto metadata = fw->result();
+            // Even when we failed to fetch information about runtime we still want to show the application
+            if (metadata.isEmpty()) {
+                onFetchMetadataFinished(installation, resource, metadata);
+            } else {
+                updateAppMetadata(resource, metadata);
+
+                auto runtime = getRuntimeForApp(resource);
+                if (!runtime || (runtime && !runtime->isInstalled())) {
+                    FlatpakFetchRemoteResourceJob *fetchRemoteResource = new FlatpakFetchRemoteResourceJob(runtimeUrl, this);
+                    connect(fetchRemoteResource, &FlatpakFetchRemoteResourceJob::jobFinished, this, [this, resource] (bool success, FlatpakResource *repoResource) {
+                        if (success) {
+                            installApplication(repoResource);
+                        }
+                        addResource(resource);
+                    });
+                    fetchRemoteResource->start();
+                    return;
+                } else {
+                    addResource(resource);
+                }
+            }
+            fw->deleteLater();
+        });
+    } else {
+        addResource(resource);
+    }
+
     return resource;
 }
 
 FlatpakResource * FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url)
 {
+    Q_ASSERT(url.isLocalFile());
     QSettings settings(url.toLocalFile(), QSettings::NativeFormat);
 
     const QString gpgKey = settings.value(QStringLiteral("Flatpak Repo/GPGKey")).toString();
@@ -376,20 +467,21 @@ FlatpakResource * FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url)
         return nullptr;
     }
 
-    AsComponent *component = as_component_new();
-    as_component_add_url(component, AS_URL_KIND_HOMEPAGE, settings.value(QStringLiteral("Flatpak Repo/Homepage")).toString().toStdString().c_str());
-    as_component_set_summary(component, settings.value(QStringLiteral("Flatpak Repo/Comment")).toString().toStdString().c_str(), nullptr);
-    as_component_set_description(component, settings.value(QStringLiteral("Flatpak Repo/Description")).toString().toStdString().c_str(), nullptr);
-    as_component_set_name(component, title.toStdString().c_str(), nullptr);
+    AppStream::Component asComponent;
+    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Repo/Homepage")).toString());
+    asComponent.setSummary(settings.value(QStringLiteral("Flatpak Repo/Comment")).toString());
+    asComponent.setDescription(settings.value(QStringLiteral("Flatpak Repo/Description")).toString());
+    asComponent.setName(title);
+    asComponent.setId(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+
     const QString iconUrl = settings.value(QStringLiteral("Flatpak Repo/Icon")).toString();
     if (!iconUrl.isEmpty()) {
-        AsIcon *icon = as_icon_new();
-        as_icon_set_kind(icon, AS_ICON_KIND_REMOTE);
-        as_icon_set_url(icon, iconUrl.toStdString().c_str());
-        as_component_add_icon(component, icon);
+        AppStream::Icon icon;
+        icon.setKind(AppStream::Icon::KindRemote);
+        icon.setUrl(QUrl(iconUrl));
+        asComponent.addIcon(icon);
     }
 
-    AppStream::Component *asComponent = new AppStream::Component(component);
     auto resource = new FlatpakResource(asComponent, preferredInstallation(), this);
     // Use metadata only for stuff which are not common for all resources
     resource->addMetadata(QStringLiteral("gpg-key"), gpgKey);
@@ -398,7 +490,7 @@ FlatpakResource * FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url)
     resource->setFlatpakName(url.fileName().remove(QStringLiteral(".flatpakrepo")));
     resource->setType(FlatpakResource::Source);
 
-    auto repo = flatpak_installation_get_remote_by_name(preferredInstallation(), resource->flatpakName().toStdString().c_str(), m_cancellable, nullptr);
+    auto repo = flatpak_installation_get_remote_by_name(preferredInstallation(), resource->flatpakName().toUtf8().constData(), m_cancellable, nullptr);
     if (!repo) {
         resource->setState(AbstractResource::State::None);
     } else {
@@ -426,7 +518,7 @@ void FlatpakBackend::addResource(FlatpakResource *resource)
     m_resources.insert(resource->uniqueId(), resource);
 }
 
-bool FlatpakBackend::compareAppFlatpakRef(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, FlatpakInstalledRef *ref)
+bool FlatpakBackend::compareAppFlatpakRef(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, FlatpakInstalledRef *ref) const
 {
     const QString arch = QString::fromUtf8(flatpak_ref_get_arch(FLATPAK_REF(ref)));
     const QString branch = QString::fromUtf8(flatpak_ref_get_branch(FLATPAK_REF(ref)));
@@ -492,14 +584,22 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
 {
     Q_ASSERT(flatpakInstallation);
 
-    g_autoptr(GPtrArray) remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, nullptr);
+    GPtrArray *remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, nullptr);
     if (!remotes) {
         return false;
     }
 
     for (uint i = 0; i < remotes->len; i++) {
         FlatpakRemote *remote = FLATPAK_REMOTE(g_ptr_array_index(remotes, i));
-        integrateRemote(flatpakInstallation, remote);
+        g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, nullptr);
+
+        QFileInfo fileInfo = QFileInfo(QString::fromUtf8(g_file_get_path(fileTimestamp)));
+        // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
+        if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
+            refreshAppstreamMetadata(flatpakInstallation, remote);
+        } else {
+            integrateRemote(flatpakInstallation, remote);
+        }
     }
 
     return true;
@@ -507,38 +607,37 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
 
 void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
 {
-    g_autoptr(GError) localError = nullptr;
-
     FlatpakSource source(remote);
     if (!source.isEnabled() || flatpak_remote_get_noenumerate(remote)) {
         return;
     }
 
     const QString appstreamDirPath = source.appstreamDir();
+    const QString appstreamIconsPath = source.appstreamDir() + QLatin1String("/icons/");
     const QString appDirFileName = appstreamDirPath + QLatin1String("/appstream.xml.gz");
     if (!QFile::exists(appDirFileName)) {
         qWarning() << "No" << appDirFileName << "appstream metadata found for" << source.name();
         return;
     }
 
-    g_autoptr(AsMetadata) metadata = as_metadata_new();
-    g_autoptr(GFile) file = g_file_new_for_path(appDirFileName.toStdString().c_str());
-    as_metadata_set_format_style (metadata, AS_FORMAT_STYLE_COLLECTION);
-    as_metadata_parse_file(metadata, file, AS_FORMAT_KIND_XML, &localError);
-    if (localError) {
-        qWarning() << "Failed to parse appstream metadata" << localError->message;
+    AppStream::Metadata metadata;
+    metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
+    AppStream::Metadata::MetadataError error = metadata.parseFile(appDirFileName, AppStream::Metadata::FormatKindXml);
+    if (error != AppStream::Metadata::MetadataErrorNoError) {
+        qWarning() << "Failed to parse appstream metadata: " << error;
         return;
     }
 
-    g_autoptr(GPtrArray) components = as_metadata_get_components(metadata);
-    for (uint i = 0; i < components->len; i++) {
-        AsComponent *component = AS_COMPONENT(g_ptr_array_index(components, i));
-        AppStream::Component *appstreamComponent = new AppStream::Component(component);
+    setFetching(true);
+    QList<AppStream::Component> components = metadata.components();
+    foreach (const AppStream::Component& component, components) {
+        AppStream::Component appstreamComponent(component);
         FlatpakResource *resource = new FlatpakResource(appstreamComponent, flatpakInstallation, this);
-        resource->setIconPath(appstreamDirPath);
+        resource->setIconPath(appstreamIconsPath);
         resource->setOrigin(source.name());
         addResource(resource);
     }
+    setFetching(false);
 }
 
 bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
@@ -551,32 +650,21 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
 
     const QDir dir(pathApps);
     if (dir.exists()) {
-        foreach (const QString &file, dir.entryList(QDir::NoDotAndDotDot | QDir::Files)) {
-            QString fnDesktop;
-            AsComponent *component;
-            g_autoptr(GError) localError = nullptr;
-            g_autoptr(GFile) desktopFile = nullptr;
-            g_autoptr(AsMetadata) metadata = as_metadata_new();
-
-            if (file == QLatin1String("mimeinfo.cache")) {
+        foreach (const auto &file, dir.entryInfoList( QDir::Files)) {
+            if (file.fileName() == QLatin1String("mimeinfo.cache")) {
                 continue;
             }
 
-            fnDesktop = pathApps + file;
-            desktopFile = g_file_new_for_path(fnDesktop.toStdString().c_str());
-            if (!desktopFile) {
-                qWarning() << "Couldn't open" << fnDesktop << " :" << localError->message;
+            const QString fnDesktop = file.absoluteFilePath();
+
+            AppStream::Metadata metadata;
+            AppStream::Metadata::MetadataError error = metadata.parseFile(fnDesktop, AppStream::Metadata::FormatKindDesktopEntry);
+            if (error != AppStream::Metadata::MetadataErrorNoError) {
+                qWarning() << "Failed to parse appstream metadata: " << error;
                 continue;
             }
 
-            as_metadata_parse_file(metadata, desktopFile, AS_FORMAT_KIND_DESKTOP_ENTRY, &localError);
-            if (localError) {
-                qWarning() << "Failed to parse appstream metadata" << localError->message;
-                continue;
-            }
-
-            component = as_metadata_get_component(metadata);
-            AppStream::Component *appstreamComponent = new AppStream::Component(component);
+            AppStream::Component appstreamComponent(metadata.component());
             FlatpakResource *resource = new FlatpakResource(appstreamComponent, flatpakInstallation, this);
 
             resource->setIconPath(pathExports);
@@ -639,12 +727,23 @@ void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
     }
 }
 
-void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation *flatpakInstallation)
+void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
 {
-    FlatpakFetchUpdatesJob *job = new FlatpakFetchUpdatesJob(flatpakInstallation);
-    connect(job, &FlatpakFetchUpdatesJob::finished, job, &FlatpakFetchUpdatesJob::deleteLater);
-    connect(job, &FlatpakFetchUpdatesJob::jobFetchUpdatesFinished, this, &FlatpakBackend::onFetchUpdatesFinished);
-    job->start();
+    auto fw = new QFutureWatcher<GPtrArray *>(this);
+    fw->setFuture(QtConcurrent::run(&m_threadPool, [installation]() -> GPtrArray * {
+        g_autoptr(GCancellable) cancellable = g_cancellable_new();
+        g_autoptr(GError) localError = nullptr;
+        GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation, cancellable, &localError);
+        if (!refs) {
+            qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
+        }
+        return refs;
+    }));
+    connect(fw, &QFutureWatcher<GPtrArray *>::finished, this, [this, installation, fw](){
+        auto refs = fw->result();
+        onFetchUpdatesFinished(installation, refs);
+        fw->deleteLater();
+    });
 }
 
 void FlatpakBackend::onFetchUpdatesFinished(FlatpakInstallation *flatpakInstallation, GPtrArray *updates)
@@ -665,11 +764,11 @@ bool FlatpakBackend::parseMetadataFromAppBundle(FlatpakResource *resource)
 {
     g_autoptr(FlatpakRef) ref = nullptr;
     g_autoptr(GError) localError = nullptr;
-    AppStream::Bundle bundle = resource->appstreamComponent()->bundle(AppStream::Bundle::KindFlatpak);
+    AppStream::Bundle bundle = resource->appstreamComponent().bundle(AppStream::Bundle::KindFlatpak);
 
     // Get arch/branch/commit/name from FlatpakRef
     if (!bundle.isEmpty()) {
-        ref = flatpak_ref_parse(bundle.id().toStdString().c_str(), &localError);
+        ref = flatpak_ref_parse(bundle.id().toUtf8().constData(), &localError);
         if (!ref) {
             qWarning() << "Failed to parse" << bundle.id() << localError->message;
             return false;
@@ -681,6 +780,64 @@ bool FlatpakBackend::parseMetadataFromAppBundle(FlatpakResource *resource)
     return true;
 }
 
+class FlatpakRefreshAppstreamMetadataJob : public QThread
+{
+    Q_OBJECT
+public:
+    FlatpakRefreshAppstreamMetadataJob(FlatpakInstallation *installation, FlatpakRemote *remote)
+        : QThread()
+        , m_cancellable(g_cancellable_new())
+        , m_installation(installation)
+        , m_remote(remote)
+    {
+        connect(this, &FlatpakRefreshAppstreamMetadataJob::finished, this, &QObject::deleteLater);
+    }
+
+    ~FlatpakRefreshAppstreamMetadataJob()
+    {
+        g_object_unref(m_cancellable);
+    }
+
+    void cancel()
+    {
+        g_cancellable_cancel(m_cancellable);
+    }
+
+    void run() override
+    {
+        g_autoptr(GError) localError = nullptr;
+
+#if FLATPAK_CHECK_VERSION(0,9,4)
+        // With Flatpak 0.9.4 we can use flatpak_installation_update_appstream_full_sync() providing progress reporting which we don't use at this moment, but still
+        // better to use newer function in case the previous one gets deprecated
+        if (!flatpak_installation_update_appstream_full_sync(m_installation, flatpak_remote_get_name(m_remote), nullptr, nullptr, nullptr, nullptr, m_cancellable, &localError)) {
+#else
+        if (!flatpak_installation_update_appstream_sync(m_installation, flatpak_remote_get_name(m_remote), nullptr, nullptr, m_cancellable, &localError)) {
+#endif
+            qWarning() << "Failed to refresh appstream metadata for " << flatpak_remote_get_name(m_remote) << ": " << (localError ? localError->message : "<no error>");
+            Q_EMIT jobRefreshAppstreamMetadataFailed();
+        } else  {
+            Q_EMIT jobRefreshAppstreamMetadataFinished(m_installation, m_remote);
+        }
+    }
+
+Q_SIGNALS:
+    void jobRefreshAppstreamMetadataFailed();
+    void jobRefreshAppstreamMetadataFinished(FlatpakInstallation *installation, FlatpakRemote *remote);
+
+private:
+    GCancellable *m_cancellable;
+    FlatpakInstallation *m_installation;
+    FlatpakRemote *m_remote;
+};
+
+void FlatpakBackend::refreshAppstreamMetadata(FlatpakInstallation *installation, FlatpakRemote *remote)
+{
+    FlatpakRefreshAppstreamMetadataJob *job = new FlatpakRefreshAppstreamMetadataJob(installation, remote);
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
+    job->start();
+}
+
 void FlatpakBackend::reloadPackageList()
 {
     setFetching(true);
@@ -742,10 +899,15 @@ bool FlatpakBackend::updateAppMetadata(FlatpakInstallation* flatpakInstallation,
     if (QFile::exists(path)) {
         return updateAppMetadata(resource, path);
     } else {
-        FlatpakFetchDataJob *job = new FlatpakFetchDataJob(flatpakInstallation, resource, FlatpakFetchDataJob::FetchMetadata);
-        connect(job, &FlatpakFetchDataJob::finished, job, &FlatpakFetchDataJob::deleteLater);
-        connect(job, &FlatpakFetchDataJob::jobFetchMetadataFinished, this, &FlatpakBackend::onFetchMetadataFinished);
-        job->start();
+        auto fw = new QFutureWatcher<QByteArray>(this);
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, flatpakInstallation, resource));
+        connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, flatpakInstallation, resource, fw]() {
+            const auto metadata = fw->result();
+            if (!metadata.isEmpty())
+                onFetchMetadataFinished(flatpakInstallation, resource, metadata);
+            fw->deleteLater();
+        });
+
         // Return false to indicate we cannot continue (right now used only in updateAppSize())
         return false;
     }
@@ -854,14 +1016,18 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
             return false;
         }
 
-        FlatpakFetchDataJob *job = new FlatpakFetchDataJob(flatpakInstallation, resource, FlatpakFetchDataJob::FetchSize);
-        connect(job, &FlatpakFetchDataJob::finished, job, &FlatpakFetchDataJob::deleteLater);
-        connect(job, &FlatpakFetchDataJob::jobFetchSizeFinished, this, &FlatpakBackend::onFetchSizeFinished);
-        connect(job, &FlatpakFetchDataJob::jobFetchSizeFailed, [resource] () {
-            resource->setPropertyState(FlatpakResource::DownloadSize, FlatpakResource::UnknownOrFailed);
-            resource->setPropertyState(FlatpakResource::InstalledSize, FlatpakResource::UnknownOrFailed);
+        auto futureWatcher = new QFutureWatcher<FlatpakRunnables::SizeInformation>(this);
+        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchFlatpakSize, flatpakInstallation, resource));
+        connect(futureWatcher, &QFutureWatcher<FlatpakRunnables::SizeInformation>::finished, this, [this, resource, futureWatcher]() {
+            auto value = futureWatcher->result();
+            if (value.valid) {
+                onFetchSizeFinished(resource, value.downloadSize, value.installedSize);
+            } else {
+                resource->setPropertyState(FlatpakResource::DownloadSize, FlatpakResource::UnknownOrFailed);
+                resource->setPropertyState(FlatpakResource::InstalledSize, FlatpakResource::UnknownOrFailed);
+            }
+            futureWatcher->deleteLater();
         });
-        job->start();
     }
 
     return true;
@@ -910,14 +1076,32 @@ int FlatpakBackend::updatesCount() const
 
 ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
-    QVector<AbstractResource*> ret;
+    if (filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakrepo")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakref"))) {
+        auto stream = new ResultsStream(QStringLiteral("FlatpakStream-http-")+filter.resourceUrl.fileName());
 
+        FlatpakFetchRemoteResourceJob *fetchResourceJob = new FlatpakFetchRemoteResourceJob(filter.resourceUrl, this);
+        connect(fetchResourceJob, &FlatpakFetchRemoteResourceJob::jobFinished, this, [fetchResourceJob, stream] (bool success, FlatpakResource *resource) {
+            if (success) {
+                stream->resourcesFound({resource});
+            }
+            stream->finish();
+            fetchResourceJob->deleteLater();
+        });
+
+        return stream;
+    } else if (!filter.resourceUrl.isEmpty() && filter.resourceUrl.scheme() != QLatin1String("appstream"))
+        return new ResultsStream(QStringLiteral("FlatpakStream-void"), {});
+
+    QVector<AbstractResource*> ret;
     foreach(AbstractResource* r, m_resources) {
         if (r->isTechnical() && filter.state != AbstractResource::Upgradeable) {
             continue;
         }
 
-        if (r->state()<filter.state)
+        if (!filter.resourceUrl.isEmpty() && filter.resourceUrl.host().compare(r->appstreamId(), Qt::CaseInsensitive) != 0)
+            continue;
+
+        if (r->state() < filter.state)
             continue;
 
         if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)) {
@@ -953,7 +1137,7 @@ AbstractReviewsBackend * FlatpakBackend::reviewsBackend() const
     return m_reviews.data();
 }
 
-void FlatpakBackend::installApplication(AbstractResource *app, const AddonList &addons)
+Transaction* FlatpakBackend::installApplication(AbstractResource *app, const AddonList &addons)
 {
     Q_UNUSED(addons);
 
@@ -964,16 +1148,18 @@ void FlatpakBackend::installApplication(AbstractResource *app, const AddonList &
         FlatpakRemote *remote = m_sources->installSource(resource);
         if (remote) {
             resource->setState(AbstractResource::Installed);
-            integrateRemote(preferredInstallation(), remote);
+            // Make sure we update appstream metadata first
+            // FIXME we have to let flatpak to return the remote as the one created by FlatpakSourcesBackend will not have appstream directory
+            refreshAppstreamMetadata(preferredInstallation(), flatpak_installation_get_remote_by_name(preferredInstallation(), flatpak_remote_get_name(remote), nullptr, nullptr));
         }
-        return;
+        return nullptr;
     }
 
     FlatpakTransaction *transaction = nullptr;
     FlatpakInstallation *installation = resource->installation();
 
     if (resource->propertyState(FlatpakResource::RequiredRuntime) == FlatpakResource::NotKnownYet && resource->type() == FlatpakResource::DesktopApp) {
-        transaction = new FlatpakTransaction(installation, resource, Transaction::InstallRole, true);
+        transaction = new FlatpakTransaction(resource, Transaction::InstallRole, true);
         connect(resource, &FlatpakResource::propertyStateChanged, [resource, transaction, this] (FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState state) {
             if (kind != FlatpakResource::RequiredRuntime) {
                 return;
@@ -990,9 +1176,9 @@ void FlatpakBackend::installApplication(AbstractResource *app, const AddonList &
     } else {
         FlatpakResource *runtime = getRuntimeForApp(resource);
         if (runtime && !runtime->isInstalled()) {
-            transaction = new FlatpakTransaction(installation, resource, runtime, Transaction::InstallRole);
+            transaction = new FlatpakTransaction(resource, runtime, Transaction::InstallRole);
         } else {
-            transaction = new FlatpakTransaction(installation, resource, Transaction::InstallRole);
+            transaction = new FlatpakTransaction(resource, Transaction::InstallRole);
         }
     }
 
@@ -1001,14 +1187,15 @@ void FlatpakBackend::installApplication(AbstractResource *app, const AddonList &
             updateAppState(installation, resource);
         }
     });
+    return transaction;
 }
 
-void FlatpakBackend::installApplication(AbstractResource *app)
+Transaction* FlatpakBackend::installApplication(AbstractResource *app)
 {
-    installApplication(app, {});
+    return installApplication(app, {});
 }
 
-void FlatpakBackend::removeApplication(AbstractResource *app)
+Transaction* FlatpakBackend::removeApplication(AbstractResource *app)
 {
     FlatpakResource *resource = qobject_cast<FlatpakResource*>(app);
 
@@ -1017,17 +1204,18 @@ void FlatpakBackend::removeApplication(AbstractResource *app)
         if (m_sources->removeSource(resource->flatpakName())) {
             resource->setState(AbstractResource::None);
         }
-        return;
+        return nullptr;
     }
 
     FlatpakInstallation *installation = resource->installation();
-    FlatpakTransaction *transaction = new FlatpakTransaction(installation, resource, Transaction::RemoveRole);
+    FlatpakTransaction *transaction = new FlatpakTransaction(resource, Transaction::RemoveRole);
 
     connect(transaction, &FlatpakTransaction::statusChanged, [this, installation, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
             updateAppSize(installation, resource);
         }
     });
+    return transaction;
 }
 
 void FlatpakBackend::checkForUpdates()
@@ -1043,7 +1231,7 @@ void FlatpakBackend::checkForUpdates()
 
 AbstractResource * FlatpakBackend::resourceForFile(const QUrl &url)
 {
-    if ((!url.path().endsWith(QLatin1String(".flatpakref")) && !url.path().endsWith(QLatin1String(".flatpak")) && !url.path().endsWith(QLatin1String(".flatpakrepo"))) || !url.isLocalFile()) {
+    if (!url.isLocalFile()) {
         return nullptr;
     }
 
@@ -1052,11 +1240,16 @@ AbstractResource * FlatpakBackend::resourceForFile(const QUrl &url)
         resource = addAppFromFlatpakBundle(url);
     } else if (url.path().endsWith(QLatin1String(".flatpakref"))) {
         resource = addAppFromFlatpakRef(url);
-    } else {
+    } else if (url.path().endsWith(QLatin1String(".flatpakrepo"))) {
         resource = addSourceFromFlatpakRepo(url);
     }
 
     return resource;
 }
 
+QString FlatpakBackend::displayName() const
+{
+    return QStringLiteral("Flatpak");
+}
+
 #include "FlatpakBackend.moc"
