@@ -29,7 +29,6 @@
 #include <resources/AbstractResource.h>
 #include <resources/StandardBackendUpdater.h>
 #include <resources/SourcesModel.h>
-#include <Transaction/TransactionModel.h>
 #include <appstream/OdrsReviewsBackend.h>
 #include <appstream/AppStreamIntegration.h>
 
@@ -65,17 +64,6 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     , m_isFetching(0)
     , m_reviews(AppStreamIntegration::global()->reviews())
 {
-    bool b = m_appdata.load();
-    reloadPackageList();
-
-    if (!b && m_packages.packages.isEmpty()) {
-        qWarning() << "Could not open the AppStream metadata pool";
-
-        QTimer::singleShot(0, this, [this]() {
-            Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
-        });
-    }
-
     QTimer* t = new QTimer(this);
     connect(t, &QTimer::timeout, this, &PackageKitBackend::refreshDatabase);
     t->setInterval(60 * 60 * 1000);
@@ -101,12 +89,29 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, &AbstractResourcesBackend::emitRatingsReady);
 
     SourcesModel::global()->addSourcesBackend(new PackageKitSourcesBackend(this));
+
+    QTimer::singleShot(0, this, &PackageKitBackend::delayedInit);
 }
 
 PackageKitBackend::~PackageKitBackend()
 {
 }
 
+void PackageKitBackend::delayedInit()
+{
+    QString error;
+    const bool b = m_appdata.load(&error);
+    reloadPackageList();
+
+    if (!b && m_packages.packages.isEmpty()) {
+        qWarning() << "Could not open the AppStream metadata pool" << error;
+
+        QTimer::singleShot(0, this, [this]() {
+            Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
+        });
+    }
+}
+
 QAction* PackageKitBackend::createActionForService(const QString &servicePath)
 {
     QAction* action = new QAction(this);
@@ -150,6 +155,9 @@ void PackageKitBackend::reloadPackageList()
     QStringList neededPackages;
     neededPackages.reserve(components.size());
     foreach(const AppStream::Component& component, components) {
+        if (component.kind() == AppStream::Component::KindFirmware)
+            continue;
+
         const auto pkgNames = component.packageNames();
         if (pkgNames.isEmpty()) {
             if (component.kind() == AppStream::Component::KindDesktopApp) {
@@ -389,12 +397,14 @@ void PackageKitBackend::refreshDatabase()
 
 ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters& filter)
 {
-    if (filter.search.isEmpty()) {
-        return new ResultsStream(QStringLiteral("PackageKitStream"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return !res->isTechnical(); }));
+    if (!filter.resourceUrl.isEmpty()) {
+        return findResourceByPackageName(filter.resourceUrl);
+    } else if (filter.search.isEmpty()) {
+        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return !res->isTechnical(); }));
     } else {
         const QList<AppStream::Component> components = m_appdata.search(filter.search);
         const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
-        auto stream = new ResultsStream(QStringLiteral("PackageKitStream"));
+        auto stream = new ResultsStream(QStringLiteral("PackageKitStream-search"));
         if (!ids.isEmpty()) {
             const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(ids);
             QTimer::singleShot(0, this, [stream, resources, this] () {
@@ -431,7 +441,7 @@ ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
         else
             pkg = m_packages.packages.value(url.host());
     }
-    return new ResultsStream(QStringLiteral("PackageKitStream"), pkg ? QVector<AbstractResource*>{pkg} : QVector<AbstractResource*>{});
+    return new ResultsStream(QStringLiteral("PackageKitStream-url"), pkg ? QVector<AbstractResource*>{pkg} : QVector<AbstractResource*>{});
 }
 
 int PackageKitBackend::updatesCount() const
@@ -439,8 +449,9 @@ int PackageKitBackend::updatesCount() const
     return m_updatesPackageId.count();
 }
 
-void PackageKitBackend::installApplication(AbstractResource* app, const AddonList& addons)
+Transaction* PackageKitBackend::installApplication(AbstractResource* app, const AddonList& addons)
 {
+    Transaction* t = nullptr;
     if(!addons.addonsToInstall().isEmpty())
     {
         QVector<AbstractResource*> appsToInstall;
@@ -452,27 +463,29 @@ void PackageKitBackend::installApplication(AbstractResource* app, const AddonLis
             appsToInstall += m_packages.packages.value(toInstall);
             Q_ASSERT(appsToInstall.last());
         }
-        new PKTransaction(appsToInstall, Transaction::ChangeAddonsRole);
+        t = new PKTransaction(appsToInstall, Transaction::ChangeAddonsRole);
     }
 
     if (!addons.addonsToRemove().isEmpty()) {
         QVector<AbstractResource*> appsToRemove = kTransform<QVector<AbstractResource*>>(addons.addonsToRemove(), [this](const QString& toRemove){ return m_packages.packages.value(toRemove); });
-        new PKTransaction(appsToRemove, Transaction::RemoveRole);
+        t = new PKTransaction(appsToRemove, Transaction::RemoveRole);
     }
 
     if (!app->isInstalled())
-        installApplication(app);
+        t = installApplication(app);
+
+    return t;
 }
 
-void PackageKitBackend::installApplication(AbstractResource* app)
+Transaction* PackageKitBackend::installApplication(AbstractResource* app)
 {
-    new PKTransaction({app}, Transaction::InstallRole);
+    return new PKTransaction({app}, Transaction::InstallRole);
 }
 
-void PackageKitBackend::removeApplication(AbstractResource* app)
+Transaction* PackageKitBackend::removeApplication(AbstractResource* app)
 {
     Q_ASSERT(!isFetching());
-    new PKTransaction({app}, Transaction::RemoveRole);
+    return new PKTransaction({app}, Transaction::RemoveRole);
 }
 
 QSet<AbstractResource*> PackageKitBackend::upgradeablePackages() const
@@ -593,5 +606,22 @@ AbstractResource * PackageKitBackend::resourceForFile(const QUrl& file)
     return nullptr;
 }
 
+static QString readDistroName()
+{
+    QProcess process;
+    process.setEnvironment({QStringLiteral("LC_ALL=C")});
+    process.start(QStringLiteral("lsb_release"), {QStringLiteral("-sd")});
+    process.waitForFinished();
+    auto output = process.readAll().trimmed();
+    if (output.startsWith('\"') && output.endsWith('\"'))
+        output = output.mid(1, output.length()-2);
+    return QString::fromLocal8Bit(output);
+}
+
+QString PackageKitBackend::displayName() const
+{
+    static const QString distro = readDistroName();
+    return distro;
+}
 
 #include "PackageKitBackend.moc"
