@@ -26,7 +26,6 @@
 #include <resources/SourcesModel.h>
 #include <Category/Category.h>
 #include <Transaction/Transaction.h>
-#include <Transaction/TransactionModel.h>
 
 #include <KAboutData>
 #include <KLocalizedString>
@@ -38,6 +37,8 @@
 #include <QTimer>
 #include <QAction>
 
+#include "utils.h"
+
 MUON_BACKEND_PLUGIN(SnapBackend)
 
 SnapBackend::SnapBackend(QObject* parent)
@@ -45,7 +46,19 @@ SnapBackend::SnapBackend(QObject* parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_reviews(new SnapReviewsBackend(this))
 {
+    {
+        auto request = m_client.connect();
+        request->runSync();
+        m_valid = request->error() == QSnapdRequest::NoError;
+        if (!m_valid) {
+            qWarning() << "snap problem at initialize:" << request->errorString();
+            return;
+        }
+    }
     connect(m_reviews, &SnapReviewsBackend::ratingsReady, this, &AbstractResourcesBackend::emitRatingsReady);
+
+    //make sure we populate the installed resources first
+    populate(m_client.list(), AbstractResource::Installed);
 }
 
 int SnapBackend::updatesCount() const
@@ -57,56 +70,59 @@ static ResultsStream* voidStream() { return new ResultsStream(QStringLiteral("Sn
 
 ResultsStream * SnapBackend::search(const AbstractResourcesBackend::Filters& filters)
 {
-    if (filters.category && filters.category->isAddons())
+    if (!filters.resourceUrl.isEmpty()) {
+        return findResourceByPackageName(filters.resourceUrl);
+    } else if (filters.category && filters.category->isAddons()) {
         return voidStream();
-    if (filters.state >= AbstractResource::Installed) {
-        return populate(m_socket.snaps(), AbstractResource::Installed);
+    } else if (filters.state >= AbstractResource::Installed) {
+        return populate(m_client.list(), AbstractResource::Installed);
     } else {
-        return populate(m_socket.find(filters.search), AbstractResource::None);
+        return populate(m_client.find(QSnapdClient::FindFlag::None, filters.search), AbstractResource::None);
     }
     return voidStream();
 }
 
 ResultsStream * SnapBackend::findResourceByPackageName(const QUrl& search)
 {
-    return search.scheme() == QLatin1String("snap") ? populate(m_socket.snapByName(search.host()), AbstractResource::Installed) : voidStream();
+    return search.scheme() == QLatin1String("snap") ? populate(m_client.find(QSnapdClient::MatchName, search.host()), AbstractResource::Installed) : voidStream();
 }
 
-ResultsStream* SnapBackend::populate(SnapJob* job, AbstractResource::State state)
+template <class T>
+ResultsStream* SnapBackend::populate(T* job, AbstractResource::State state)
 {
     auto stream = new ResultsStream(QStringLiteral("Snap-populate"));
 
-    connect(job, &SnapJob::finished, stream, [stream, this, state](SnapJob* job) {
-        if (!job->isSuccessful()) {
-            stream->finish();
-            return;
-        }
-
-        const auto snaps = job->result().toArray();
+    connect(job, &QSnapdFindRequest::complete, stream, [stream, this, state, job]() {
+        QSet<SnapResource*> higher = kFilter<QSet<SnapResource*>>(m_resources, [state](AbstractResource* res){ return res->state()>=state; });
 
         QVector<AbstractResource*> ret;
         QSet<SnapResource*> resources;
-        for(const auto& snap: snaps) {
-            const auto snapObj = snap.toObject();
-            const auto snapid = snapObj.value(QLatin1String("name")).toString();
-            SnapResource* res = m_resources.value(snapid);
+        for (int i=0, c=job->snapCount(); i<c; ++i) {
+            QSharedPointer<QSnapdSnap> snap(job->snap(i));
+            const auto snapname = snap->name();
+            SnapResource* res = m_resources.value(snapname);
             if (!res) {
-                res = new SnapResource(snapObj, state, this);
-                Q_ASSERT(res->packageName() == snapid);
+                res = new SnapResource(snap, state, this);
+                Q_ASSERT(res->packageName() == snapname);
                 resources += res;
+            } else {
+                res->setState(state);
+                higher.remove(res);
             }
             ret += res;
         }
 
-        if (!resources.isEmpty()) {
-            foreach(SnapResource* res, resources)
-                m_resources[res->packageName()] = res;
+        foreach(SnapResource* res, resources)
+            m_resources[res->packageName()] = res;
+        for(auto res: higher) {
+            res->setState(AbstractResource::None);
         }
+
         if (!ret.isEmpty())
             stream->resourcesFound(ret);
         stream->finish();
     });
-    job->start();
+    job->runAsync();
     return stream;
 }
 
@@ -130,26 +146,32 @@ AbstractReviewsBackend* SnapBackend::reviewsBackend() const
     return m_reviews;
 }
 
-void SnapBackend::installApplication(AbstractResource* app, const AddonList& addons)
+Transaction* SnapBackend::installApplication(AbstractResource* app, const AddonList& addons)
 {
     Q_ASSERT(addons.isEmpty());
-    installApplication(app);
+    return installApplication(app);
 }
 
-void SnapBackend::installApplication(AbstractResource* _app)
+Transaction* SnapBackend::installApplication(AbstractResource* _app)
 {
-	TransactionModel *transModel = TransactionModel::global();
     auto app = qobject_cast<SnapResource*>(_app);
-    auto job = m_socket.snapAction(app->packageName(), SnapSocket::Install);
-	transModel->addTransaction(new SnapTransaction(app, job, &m_socket, Transaction::InstallRole));
+	return new SnapTransaction(app, m_client.install(app->packageName()), Transaction::InstallRole);
 }
 
-void SnapBackend::removeApplication(AbstractResource* _app)
+Transaction* SnapBackend::removeApplication(AbstractResource* _app)
 {
-	TransactionModel *transModel = TransactionModel::global();
     auto app = qobject_cast<SnapResource*>(_app);
-    auto job = m_socket.snapAction(app->packageName(), SnapSocket::Remove);
-	transModel->addTransaction(new SnapTransaction(app, job, &m_socket, Transaction::RemoveRole));
+	return new SnapTransaction(app, m_client.remove(app->packageName()), Transaction::RemoveRole);
+}
+
+QString SnapBackend::displayName() const
+{
+    return QStringLiteral("Snap");
+}
+
+void SnapBackend::refreshStates()
+{
+    populate(m_client.list(), AbstractResource::Installed);
 }
 
 #include "SnapBackend.moc"
