@@ -63,13 +63,7 @@ bool AbstractResource::isInstalled()
 
 void AbstractResource::fetchScreenshots()
 {
-    QList<QUrl> thumbs, screens;
-    QUrl thumbnail = thumbnailUrl();
-    if(!thumbnail.isEmpty()) {
-        thumbs << thumbnail;
-        screens << screenshotUrl();
-    }
-    emit screenshotsFetched(thumbs, screens);
+    emit screenshotsFetched({}, {});
 }
 
 QStringList AbstractResource::mimetypes() const
@@ -93,11 +87,6 @@ QString AbstractResource::status()
     return QString();
 }
 
-bool AbstractResource::isFromSecureOrigin() const
-{
-    return false;
-}
-
 QString AbstractResource::sizeDescription()
 {
     return KFormat().formatByteSize(size());
@@ -223,3 +212,14 @@ QString AbstractResource::categoryDisplay() const
     ret.sort();
     return ret.join(QStringLiteral(", "));
 }
+
+QUrl AbstractResource::url() const
+{
+    const QString asid = appstreamId();
+    return asid.isEmpty() ? QUrl(backend()->name() + QStringLiteral("://") + packageName()) : QUrl(QStringLiteral("appstream://") + asid);
+}
+
+QString AbstractResource::displayOrigin() const
+{
+    return i18nc("origin (backend name)", "%1 (%2)", origin(), backend()->displayName());
+}
