@@ -56,13 +56,12 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QStringList category READ categories CONSTANT)
     Q_PROPERTY(bool isTechnical READ isTechnical CONSTANT)
     Q_PROPERTY(QUrl homepage READ homepage CONSTANT)
-    Q_PROPERTY(QUrl thumbnailUrl READ thumbnailUrl CONSTANT)
-    Q_PROPERTY(QUrl screenshotUrl READ screenshotUrl CONSTANT)
     Q_PROPERTY(bool canUpgrade READ canUpgrade NOTIFY stateChanged)
     Q_PROPERTY(bool isInstalled READ isInstalled NOTIFY stateChanged)
     Q_PROPERTY(QString license READ license CONSTANT)
     Q_PROPERTY(QString longDescription READ longDescription CONSTANT)
     Q_PROPERTY(QString origin READ origin CONSTANT)
+    Q_PROPERTY(QString displayOrigin READ displayOrigin CONSTANT)
     Q_PROPERTY(int size READ size NOTIFY sizeChanged)
     Q_PROPERTY(QString sizeDescription READ sizeDescription NOTIFY sizeChanged)
     Q_PROPERTY(QString installedVersion READ installedVersion NOTIFY stateChanged)
@@ -73,6 +72,7 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(Rating* rating READ rating NOTIFY ratingFetched)
     Q_PROPERTY(QString appstreamId READ appstreamId CONSTANT)
     Q_PROPERTY(QString categoryDisplay READ categoryDisplay CONSTANT)
+    Q_PROPERTY(QUrl url READ url CONSTANT)
     public:
         /**
          * This describes the state of the resource
@@ -129,9 +129,6 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
 
         virtual bool isTechnical() const;
 
-        virtual QUrl thumbnailUrl() = 0;
-        virtual QUrl screenshotUrl() = 0;
-
         virtual int size() = 0;
         virtual QString sizeDescription();
         virtual QString license() = 0;
@@ -141,13 +138,13 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
         virtual QString longDescription() = 0;
 
         virtual QString origin() const = 0;
+        QString displayOrigin() const;
         virtual QString section() = 0;
 
         ///@returns what kind of mime types the resource can consume
         virtual QStringList mimetypes() const;
 
         virtual QList<PackageState> addonsInformation() = 0;
-        bool isFromSecureOrigin() const;
 
         virtual QStringList extends() const;
 
@@ -186,6 +183,11 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
 
         QSet<Category*> categoryObjects() const;
 
+        /**
+         * @returns a url that uniquely identifies the application
+         */
+        virtual QUrl url() const;
+
     public Q_SLOTS:
         virtual void fetchScreenshots();
         virtual void fetchChangelog() = 0;
