@@ -74,6 +74,7 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(QString name READ name CONSTANT)
+    Q_PROPERTY(QString displayName READ displayName CONSTANT)
     Q_PROPERTY(AbstractReviewsBackend* reviewsBackend READ reviewsBackend CONSTANT)
     Q_PROPERTY(int updatesCount READ updatesCount NOTIFY updatesCountChanged)
     Q_PROPERTY(bool isFetching READ isFetching NOTIFY fetchingChanged)
@@ -96,7 +97,9 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
             QString mimetype;
             QString search;
             QString extends;
+            QUrl resourceUrl;
             QHash<QByteArray, QVariant> roles;
+            bool allBackends = false;
 
             bool shouldFilter(AbstractResource* res) const;
             void filterJustInCase(QVector<AbstractResource*>& input) const;
@@ -158,6 +161,8 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
         /** @internal only to be used by the factory */
         void setName(const QString& name);
 
+        virtual QString displayName() const = 0;
+
         /**
          * emits a change for all rating properties
          */
@@ -170,34 +175,34 @@ class DISCOVERCOMMON_EXPORT AbstractResourcesBackend : public QObject
          */
         virtual QVector<Category*> category() const { return {}; }
 
+        virtual bool hasApplications() const { return false; }
+
     public Q_SLOTS:
         /**
          * This gets called when the backend should install an application.
-         * The AbstractResourcesBackend should create a Transaction object, which
-         * will provide Muon with information like the status and progress of a transaction,
-         * and add it to the TransactionModel with the following line:
-         * \code
-         * TransactionModel::global()->addTransaction(transaction);
-         * \endcode
-         * where transaction is the newly created Transaction.
+         * The AbstractResourcesBackend should create a Transaction object, is returned and
+         * will be included in the TransactionModel
          * @param app the application to be installed
          * @param addons the addons which should be installed with the application
+         * @returns the Transaction that keeps track of the installation process
          */
-        virtual void installApplication(AbstractResource *app, const AddonList& addons) = 0;
+        virtual Transaction* installApplication(AbstractResource *app, const AddonList& addons) = 0;
         
         /**
          * Overloaded function, which simply does the same, except not installing any addons.
          */
-        virtual void installApplication(AbstractResource *app);
+        virtual Transaction* installApplication(AbstractResource *app);
         
         /**
          * This gets called when the backend should remove an application.
-         * Like in the installApplication() method, the AbstractResourcesBackend should
-         * create a Transaction object and add it to the TransactionModel.
+         * Like in the installApplication() method, we'll return the Transaction
+         * responsible for the removal.
+         *
          * @see installApplication
          * @param app the application to be removed
+         * @returns the Transaction that keeps track of the removal process
          */
-        virtual void removeApplication(AbstractResource *app) = 0;
+        virtual Transaction* removeApplication(AbstractResource *app) = 0;
 
         /**
          * Notifies the backend that the user wants the information to be up to date
