@@ -32,25 +32,20 @@ class QAction;
 class AbstractBackendUpdater;
 class ResourcesModel;
 class QDBusInterface;
-class Transaction;
+class UpdateTransaction;
 
 class DISCOVERCOMMON_EXPORT ResourcesUpdatesModel : public QStandardItemModel
 {
     Q_OBJECT
-    Q_PROPERTY(qreal progress READ progress NOTIFY progressChanged)
-    Q_PROPERTY(bool isCancelable READ isCancelable NOTIFY cancelableChanged)
     Q_PROPERTY(bool isProgressing READ isProgressing NOTIFY progressingChanged)
     Q_PROPERTY(QDateTime lastUpdate READ lastUpdate NOTIFY progressingChanged)
     Q_PROPERTY(qint64 secsToLastUpdate READ secsToLastUpdate NOTIFY progressingChanged)
     public:
         explicit ResourcesUpdatesModel(QObject* parent = nullptr);
-        
-        qreal progress() const;
+
         quint64 downloadSpeed() const;
         Q_SCRIPTABLE void prepare();
 
-        ///checks if any of them is cancelable
-        bool isCancelable() const;
         bool isProgressing() const;
         QList<AbstractResource*> toUpdate() const;
         QDateTime lastUpdate() const;
@@ -62,28 +57,25 @@ class DISCOVERCOMMON_EXPORT ResourcesUpdatesModel : public QStandardItemModel
 
     Q_SIGNALS:
         void downloadSpeedChanged();
-        void progressChanged();
-        void cancelableChanged();
-        void progressingChanged(bool progressing);
+        void progressingChanged();
         void finished();
         void resourceProgressed(AbstractResource* resource, qreal progress);
 
     public Q_SLOTS:
-        void cancel();
         void updateAll();
 
     private Q_SLOTS:
         void updaterDestroyed(QObject* obj);
         void message(const QString& msg);
-        void slotProgressingChanged();
 
     private:
         void init();
         void updateFinished();
+        void setTransaction(UpdateTransaction* transaction);
 
         QVector<AbstractBackendUpdater*> m_updaters;
         bool m_lastIsProgressing;
-        QPointer<Transaction> m_transaction;
+        QPointer<UpdateTransaction> m_transaction;
 };
 
 #endif // RESOURCESUPDATESMODEL_H
